@@ -1,5 +1,5 @@
 /* ldlang.h - linker command language support
-   Copyright (C) 1991-2017 Free Software Foundation, Inc.
+   Copyright (C) 1991-2020 Free Software Foundation, Inc.
 
    This file is part of the GNU Binutils.
 
@@ -68,26 +68,27 @@ typedef struct memory_region_struct
 
 enum statement_enum
 {
-  lang_output_section_statement_enum,
-  lang_assignment_statement_enum,
-  lang_input_statement_enum,
   lang_address_statement_enum,
-  lang_wild_statement_enum,
-  lang_input_section_enum,
-  lang_object_symbols_statement_enum,
-  lang_fill_statement_enum,
+  lang_assignment_statement_enum,
   lang_data_statement_enum,
-  lang_reloc_statement_enum,
-  lang_target_statement_enum,
+  lang_fill_statement_enum,
+  lang_group_statement_enum,
+  lang_input_section_enum,
+  lang_input_statement_enum,
+  lang_insert_statement_enum,
+  lang_output_section_statement_enum,
   lang_output_statement_enum,
   lang_padding_statement_enum,
-  lang_group_statement_enum,
-  lang_insert_statement_enum,
-  lang_constructors_statement_enum
+  lang_reloc_statement_enum,
+  lang_target_statement_enum,
+  lang_wild_statement_enum,
+  lang_constructors_statement_enum,
+  lang_object_symbols_statement_enum
 };
 
 typedef struct lang_statement_header_struct
 {
+  /* Next pointer for statement_list statement list.  */
   union lang_statement_union *next;
   enum statement_enum type;
 } lang_statement_header_type;
@@ -115,6 +116,7 @@ typedef struct lang_output_statement_struct
 enum section_type
 {
   normal_section,
+  first_overlay_section,
   overlay_section,
   noload_section,
   noalloc_section
@@ -143,6 +145,8 @@ typedef struct lang_output_section_statement_struct
   fill_type *fill;
   union etree_union *addr_tree;
   union etree_union *load_base;
+  union etree_union *section_alignment;
+  union etree_union *subsection_alignment;
 
   /* If non-null, an expression to evaluate after setting the section's
      size.  The expression is evaluated inside REGION (above) with '.'
@@ -153,8 +157,6 @@ typedef struct lang_output_section_statement_struct
   lang_output_section_phdr_list *phdrs;
 
   unsigned int block_value;
-  int subsection_alignment;	/* Alignment of components.  */
-  int section_alignment;	/* Alignment of start of section.  */
   int constraint;
   flagword flags;
   enum section_type sectype;
@@ -171,16 +173,6 @@ typedef struct lang_output_section_statement_struct
   unsigned int align_lma_with_input : 1;
 } lang_output_section_statement_type;
 
-typedef struct
-{
-  lang_statement_header_type header;
-} lang_common_statement_type;
-
-typedef struct
-{
-  lang_statement_header_type header;
-} lang_object_symbols_statement_type;
-
 typedef struct
 {
   lang_statement_header_type header;
@@ -304,14 +296,15 @@ typedef struct lang_input_statement_struct
 
   bfd *the_bfd;
 
+  ctf_archive_t *the_ctf;
+
   struct flag_info *section_flag_list;
 
-  /* Point to the next file - whatever it is, wanders up and down
-     archives */
-  union lang_statement_union *next;
+  /* Next pointer for file_chain statement list.  */
+  struct lang_input_statement_struct *next;
 
-  /* Point to the next file, but skips archive contents.  */
-  union lang_statement_union *next_real_file;
+  /* Next pointer for input_file_chain statement list.  */
+  struct lang_input_statement_struct *next_real_file;
 
   const char *target;
 
@@ -338,8 +331,12 @@ typedef struct input_section_userdata_struct
   unsigned long map_symbol_def_count;
 } input_section_userdata_type;
 
-#define get_userdata(x) ((x)->userdata)
-
+static inline bfd_boolean
+bfd_input_just_syms (const bfd *abfd)
+{
+  lang_input_statement_type *is = bfd_usrdata (abfd);
+  return is != NULL && is->flags.just_syms;
+}
 
 typedef struct lang_wild_statement_struct lang_wild_statement_type;
 
@@ -418,22 +415,20 @@ typedef struct
 typedef union lang_statement_union
 {
   lang_statement_header_type header;
-  lang_wild_statement_type wild_statement;
-  lang_data_statement_type data_statement;
-  lang_reloc_statement_type reloc_statement;
   lang_address_statement_type address_statement;
-  lang_output_section_statement_type output_section_statement;
   lang_assignment_statement_type assignment_statement;
-  lang_input_statement_type input_statement;
-  lang_target_statement_type target_statement;
-  lang_output_statement_type output_statement;
-  lang_input_section_type input_section;
-  lang_common_statement_type common_statement;
-  lang_object_symbols_statement_type object_symbols_statement;
+  lang_data_statement_type data_statement;
   lang_fill_statement_type fill_statement;
-  lang_padding_statement_type padding_statement;
   lang_group_statement_type group_statement;
+  lang_input_section_type input_section;
+  lang_input_statement_type input_statement;
   lang_insert_statement_type insert_statement;
+  lang_output_section_statement_type output_section_statement;
+  lang_output_statement_type output_statement;
+  lang_padding_statement_type padding_statement;
+  lang_reloc_statement_type reloc_statement;
+  lang_target_statement_type target_statement;
+  lang_wild_statement_type wild_statement;
 } lang_statement_union_type;
 
 /* This structure holds information about a program header, from the
@@ -501,7 +496,7 @@ extern struct lang_phdr *lang_phdr_list;
 extern struct lang_nocrossrefs *nocrossref_list;
 extern const char *output_target;
 extern lang_output_section_statement_type *abs_output_section;
-extern lang_statement_list_type lang_output_section_statement;
+extern lang_statement_list_type lang_os_list;
 extern struct lang_input_statement_flags input_flags;
 extern bfd_boolean lang_has_input_file;
 extern lang_statement_list_type *stat_ptr;
@@ -566,9 +561,6 @@ extern void lang_float
 extern void lang_leave_output_section_statement
   (fill_type *, const char *, lang_output_section_phdr_list *,
    const char *);
-extern void lang_statement_append
-  (lang_statement_list_type *, lang_statement_union_type *,
-   lang_statement_union_type **);
 extern void lang_for_each_input_file
   (void (*dothis) (lang_input_statement_type *));
 extern void lang_for_each_file
@@ -583,8 +575,8 @@ extern asection *section_for_dot
 #define LANG_FOR_EACH_INPUT_STATEMENT(statement)			\
   lang_input_statement_type *statement;					\
   for (statement = (lang_input_statement_type *) file_chain.head;	\
-       statement != (lang_input_statement_type *) NULL;			\
-       statement = (lang_input_statement_type *) statement->next)	\
+       statement != NULL;						\
+       statement = statement->next)
 
 #define lang_output_section_find(NAME) \
   lang_output_section_statement_lookup (NAME, 0, FALSE)
@@ -687,6 +679,12 @@ extern void add_excluded_libs (const char *);
 extern bfd_boolean load_symbols
   (lang_input_statement_type *, lang_statement_list_type *);
 
+struct elf_sym_strtab;
+struct elf_strtab_hash;
+extern void ldlang_ctf_apply_strsym
+  (struct elf_sym_strtab *, bfd_size_type, struct elf_strtab_hash *);
+extern void ldlang_write_ctf_late
+  (void);
 extern bfd_boolean
 ldlang_override_segment_assignment
   (struct bfd_link_info *, bfd *, asection *, asection *, bfd_boolean);
